@@ -19,7 +19,7 @@
 #include <asm/string.h>
 #include <asm/errno.h>
 
-#include "ieee80211.h"
+#include <net/ieee80211.h>
 
 MODULE_AUTHOR("Jouni Malinen");
 MODULE_DESCRIPTION("HostAP crypto");
@@ -43,7 +43,9 @@ void ieee80211_crypt_deinit_entries(struct ieee80211_device *ieee,
 {
 	struct list_head *ptr, *n;
 	struct ieee80211_crypt_data *entry;
+	unsigned long flags;
 
+	spin_lock_irqsave(&ieee->lock, flags);
 	for (ptr = ieee->crypt_deinit_list.next, n = ptr->next;
 	     ptr != &ieee->crypt_deinit_list; ptr = n, n = ptr->next) {
 		entry = list_entry(ptr, struct ieee80211_crypt_data, list);
@@ -59,14 +61,13 @@ void ieee80211_crypt_deinit_entries(struct ieee80211_device *ieee,
 		}
 		kfree(entry);
 	}
+	spin_unlock_irqrestore(&ieee->lock, flags);
 }
 
 void ieee80211_crypt_deinit_handler(unsigned long data)
 {
 	struct ieee80211_device *ieee = (struct ieee80211_device *)data;
-	unsigned long flags;
 
-	spin_lock_irqsave(&ieee->lock, flags);
 	ieee80211_crypt_deinit_entries(ieee, 0);
 	if (!list_empty(&ieee->crypt_deinit_list)) {
 		printk(KERN_DEBUG "%s: entries remaining in delayed crypt "
@@ -74,7 +75,6 @@ void ieee80211_crypt_deinit_handler(unsigned long data)
 		ieee->crypt_deinit_timer.expires = jiffies + HZ;
 		add_timer(&ieee->crypt_deinit_timer);
 	}
-	spin_unlock_irqrestore(&ieee->lock, flags);
 
 }
 
@@ -207,17 +207,23 @@ static struct ieee80211_crypto_ops ieee80211_crypt_null = {
 
 static int __init ieee80211_crypto_init(void)
 {
+	int ret = -ENOMEM;
+
 	hcrypt = kmalloc(sizeof(*hcrypt), GFP_KERNEL);
-	if (hcrypt == NULL)
-		return -ENOMEM;
+	if (!hcrypt)
+		goto out;
 
 	memset(hcrypt, 0, sizeof(*hcrypt));
 	INIT_LIST_HEAD(&hcrypt->algs);
 	spin_lock_init(&hcrypt->lock);
 
-	(void) ieee80211_register_crypto_ops(&ieee80211_crypt_null);
-
-	return 0;
+	ret = ieee80211_register_crypto_ops(&ieee80211_crypt_null);
+	if (ret < 0) {
+		kfree(hcrypt);
+		hcrypt = NULL;
+	}
+out:
+	return ret;
 }
 
 
