@@ -35,7 +35,7 @@
 #include <asm/uaccess.h>
 #include <linux/ctype.h>
 
-#include "ieee80211.h"
+#include <net/ieee80211.h>
 
 static inline void ieee80211_monitor_rx(struct ieee80211_device *ieee,
 					struct sk_buff *skb,
@@ -230,7 +230,6 @@ static unsigned char bridge_tunnel_header[] =
 { 0xaa, 0xaa, 0x03, 0x00, 0x00, 0xf8 };
 /* No encapsulation header if EtherType < 0x600 (=length) */
 
-#ifdef CONFIG_IEEE80211_CRYPT
 /* Called by ieee80211_rx_frame_decrypt */
 static int ieee80211_is_eapol_frame(struct ieee80211_device *ieee,
 				    struct sk_buff *skb)
@@ -285,7 +284,7 @@ ieee80211_rx_frame_decrypt(struct ieee80211_device* ieee, struct sk_buff *skb,
 	hdr = (struct ieee80211_hdr *) skb->data;
 	hdrlen = ieee80211_get_hdrlen(le16_to_cpu(hdr->frame_ctl));
 
-#ifdef CONFIG_IEEE80211_WPA
+#ifdef CONFIG_IEEE80211_CRYPT_TKIP
 	if (ieee->tkip_countermeasures &&
 	    strcmp(crypt->ops->name, "TKIP") == 0) {
 		if (net_ratelimit()) {
@@ -305,10 +304,10 @@ ieee80211_rx_frame_decrypt(struct ieee80211_device* ieee, struct sk_buff *skb,
 			"decryption failed (SA=" MAC_FMT
 			") res=%d\n", MAC_ARG(hdr->addr2), res);
 		if (res == -2)
-			IEEE80211_DEBUG_DROP("WEP decryption failed ICV "
+			IEEE80211_DEBUG_DROP("Decryption failed ICV "
 					     "mismatch (key %d)\n",
 					     skb->data[hdrlen + 3] >> 6);
-		ieee->ieee_stats.rx_discards_wep_undecryptable++;
+		ieee->ieee_stats.rx_discards_undecryptable++;
 		return -1;
 	}
 
@@ -342,7 +341,6 @@ ieee80211_rx_frame_decrypt_msdu(struct ieee80211_device* ieee, struct sk_buff *s
 
 	return 0;
 }
-#endif /* CONFIG_IEEE80211_CRYPT */
 
 
 /* All received frames are sent to this function. @skb contains the frame in
@@ -369,10 +367,8 @@ int ieee80211_rx(struct ieee80211_device *ieee, struct sk_buff *skb,
 #endif
 	u8 dst[ETH_ALEN];
 	u8 src[ETH_ALEN];
-#ifdef CONFIG_IEEE80211_CRYPT
 	struct ieee80211_crypt_data *crypt = NULL;
 	int keyidx = 0;
-#endif
 
 	hdr = (struct ieee80211_hdr *)skb->data;
 	stats = &ieee->stats;
@@ -390,35 +386,48 @@ int ieee80211_rx(struct ieee80211_device *ieee, struct sk_buff *skb,
 	frag = WLAN_GET_SEQ_FRAG(sc);
 	hdrlen = ieee80211_get_hdrlen(fc);
 
-#ifdef NOT_YET
-#if WIRELESS_EXT > 15
 	/* Put this code here so that we avoid duplicating it in all
 	 * Rx paths. - Jean II */
 #ifdef IW_WIRELESS_SPY		/* defined in iw_handler.h */
 	/* If spy monitoring on */
-	if (iface->spy_data.spy_number > 0) {
+	if (ieee->spy_data.spy_number > 0) {
 		struct iw_quality wstats;
-		wstats.level = rx_stats->signal;
-		wstats.noise = rx_stats->noise;
-		wstats.updated = 6;	/* No qual value */
+
+		wstats.updated = 0;
+		if (rx_stats->mask & IEEE80211_STATMASK_RSSI) {
+			wstats.level = rx_stats->rssi;
+			wstats.updated |= IW_QUAL_LEVEL_UPDATED;
+		} else
+			wstats.updated |= IW_QUAL_LEVEL_INVALID;
+
+		if (rx_stats->mask & IEEE80211_STATMASK_NOISE) {
+			wstats.noise = rx_stats->noise;
+			wstats.updated |= IW_QUAL_NOISE_UPDATED;
+		} else
+			wstats.updated |= IW_QUAL_NOISE_INVALID;
+
+		if (rx_stats->mask & IEEE80211_STATMASK_SIGNAL) {
+			wstats.qual = rx_stats->signal;
+			wstats.updated |= IW_QUAL_QUAL_UPDATED;
+		} else
+			wstats.updated |= IW_QUAL_QUAL_INVALID;
+
 		/* Update spy records */
-		wireless_spy_update(dev, hdr->addr2, &wstats);
+		wireless_spy_update(ieee->dev, hdr->addr2, &wstats);
 	}
 #endif /* IW_WIRELESS_SPY */
-#endif /* WIRELESS_EXT > 15 */
+
+#ifdef NOT_YET
 	hostap_update_rx_stats(local->ap, hdr, rx_stats);
 #endif
 
-#if WIRELESS_EXT > 15
 	if (ieee->iw_mode == IW_MODE_MONITOR) {
 		ieee80211_monitor_rx(ieee, skb, rx_stats);
 		stats->rx_packets++;
 		stats->rx_bytes += skb->len;
 		return 1;
 	}
-#endif
 
-#ifdef CONFIG_IEEE80211_CRYPT
 	if (ieee->host_decrypt) {
 		int idx = 0;
 		if (skb->len >= hdrlen + 3)
@@ -450,14 +459,13 @@ int ieee80211_rx(struct ieee80211_device *ieee, struct sk_buff *skb,
 			 * frames from other than current BSS, so just drop the
 			 * frames silently instead of filling system log with
 			 * these reports. */
-			IEEE80211_DEBUG_DROP("WEP decryption failed (not set)"
+			IEEE80211_DEBUG_DROP("Decryption failed (not set)"
 					     " (SA=" MAC_FMT ")\n",
 					     MAC_ARG(hdr->addr2));
-			ieee->ieee_stats.rx_discards_wep_undecryptable++;
+			ieee->ieee_stats.rx_discards_undecryptable++;
 			goto rx_dropped;
 		}
 	}
-#endif /* CONFIG_IEEE80211_CRYPT */
 
 #ifdef NOT_YET
 	if (type != WLAN_FC_TYPE_DATA) {
@@ -563,11 +571,10 @@ int ieee80211_rx(struct ieee80211_device *ieee, struct sk_buff *skb,
 
 	/* skb: hdr + (possibly fragmented, possibly encrypted) payload */
 
-#ifdef CONFIG_IEEE80211_CRYPT
 	if (ieee->host_decrypt && (fc & IEEE80211_FCTL_WEP) &&
 	    (keyidx = ieee80211_rx_frame_decrypt(ieee, skb, crypt)) < 0)
 		goto rx_dropped;
-#endif
+
 	hdr = (struct ieee80211_hdr *) skb->data;
 
 	/* skb: hdr + (possibly fragmented) plaintext payload */
@@ -628,7 +635,6 @@ int ieee80211_rx(struct ieee80211_device *ieee, struct sk_buff *skb,
 
 	/* skb: hdr + (possible reassembled) full MSDU payload; possibly still
 	 * encrypted/authenticated */
-#ifdef CONFIG_IEEE80211_CRYPT
 	if (ieee->host_decrypt && (fc & IEEE80211_FCTL_WEP) &&
 	    ieee80211_rx_frame_decrypt_msdu(ieee, skb, keyidx, crypt))
 		goto rx_dropped;
@@ -673,7 +679,6 @@ int ieee80211_rx(struct ieee80211_device *ieee, struct sk_buff *skb,
 			MAC_ARG(hdr->addr2));
 		goto rx_dropped;
 	}
-#endif /* CONFIG_IEEE80211_CRYPT */
 
 	/* skb: hdr + (possible reassembled) full plaintext payload */
 
@@ -831,11 +836,11 @@ static inline int ieee80211_network_init(
 
 	/* Pull out fixed field data */
 	memcpy(network->bssid, beacon->header.addr3, ETH_ALEN);
-	network->capability = beacon->capability;
+	network->capability = le16_to_cpu(beacon->capability); //IXP425
 	network->last_scanned = jiffies;
-	network->time_stamp[0] = beacon->time_stamp[0];
-	network->time_stamp[1] = beacon->time_stamp[1];
-	network->beacon_interval = beacon->beacon_interval;
+	network->time_stamp[0] = le32_to_cpu(beacon->time_stamp[0]); //IXP425
+	network->time_stamp[1] = le32_to_cpu(beacon->time_stamp[1]); //IXP425
+	network->beacon_interval = le16_to_cpu(beacon->beacon_interval); //IXP425
 	/* Where to pull this? beacon->listen_interval;*/
 	network->listen_interval = 0x0A;
 	network->rates_len = network->rates_ex_len = 0;
@@ -850,10 +855,8 @@ static inline int ieee80211_network_init(
 	} else
 		network->flags |= NETWORK_HAS_CCK;
 
-#ifdef CONFIG_IEEE80211_WPA
  	network->wpa_ie_len = 0;
  	network->rsn_ie_len = 0;
-#endif /* CONFIG_IEEE80211_WPA */
 
  	info_element = &beacon->info_element;
 	left = stats->len - ((void *)info_element - (void *)beacon);
@@ -957,7 +960,6 @@ static inline int ieee80211_network_init(
 			IEEE80211_DEBUG_SCAN("MFIE_TYPE_CHALLENGE: ignored\n");
 			break;
 
-#ifdef CONFIG_IEEE80211_WPA
 		case MFIE_TYPE_GENERIC:
 			IEEE80211_DEBUG_SCAN("MFIE_TYPE_GENERIC: %d bytes\n",
 					     info_element->len);
@@ -981,7 +983,6 @@ static inline int ieee80211_network_init(
 			memcpy(network->rsn_ie, info_element,
 			       network->rsn_ie_len);
 			break;
-#endif
 
 		default:
 			IEEE80211_DEBUG_SCAN("unsupported IE %d\n",
@@ -996,9 +997,9 @@ static inline int ieee80211_network_init(
   	}
 
 	network->mode = 0;
-	if (stats->freq == IEEE80211_52GHZ_BAND) {
+	if (stats->freq == IEEE80211_52GHZ_BAND)
 		network->mode = IEEE_A;
-	} else {
+	else {
 		if (network->flags & NETWORK_HAS_OFDM)
 			network->mode |= IEEE_G;
 		if (network->flags & NETWORK_HAS_CCK)
@@ -1053,12 +1054,10 @@ static inline void update_network(struct ieee80211_network *dst,
 	dst->listen_interval = src->listen_interval;
 	dst->atim_window = src->atim_window;
 
-#ifdef CONFIG_IEEE80211_WPA
 	memcpy(dst->wpa_ie, src->wpa_ie, src->wpa_ie_len);
 	dst->wpa_ie_len = src->wpa_ie_len;
 	memcpy(dst->rsn_ie, src->rsn_ie, src->rsn_ie_len);
 	dst->rsn_ie_len = src->rsn_ie_len;
-#endif /* CONFIG_IEEE80211_WPA */
 
 	dst->last_scanned = jiffies;
 	/* dst->last_associate is not overwritten */
@@ -1075,34 +1074,35 @@ static inline void ieee80211_process_probe_response(
 #ifdef CONFIG_IEEE80211_DEBUG
 	struct ieee80211_info_element *info_element = &beacon->info_element;
 #endif
+	unsigned long flags;
 
 	IEEE80211_DEBUG_SCAN(
 		"'%s' (" MAC_FMT "): %c%c%c%c %c%c%c%c-%c%c%c%c %c%c%c%c\n",
 		escape_essid(info_element->data, info_element->len),
 		MAC_ARG(beacon->header.addr3),
-		(beacon->capability & BIT(0xf)) ? '1' : '0',
-		(beacon->capability & BIT(0xe)) ? '1' : '0',
-		(beacon->capability & BIT(0xd)) ? '1' : '0',
-		(beacon->capability & BIT(0xc)) ? '1' : '0',
-		(beacon->capability & BIT(0xb)) ? '1' : '0',
-		(beacon->capability & BIT(0xa)) ? '1' : '0',
-		(beacon->capability & BIT(0x9)) ? '1' : '0',
-		(beacon->capability & BIT(0x8)) ? '1' : '0',
-		(beacon->capability & BIT(0x7)) ? '1' : '0',
-		(beacon->capability & BIT(0x6)) ? '1' : '0',
-		(beacon->capability & BIT(0x5)) ? '1' : '0',
-		(beacon->capability & BIT(0x4)) ? '1' : '0',
-		(beacon->capability & BIT(0x3)) ? '1' : '0',
-		(beacon->capability & BIT(0x2)) ? '1' : '0',
-		(beacon->capability & BIT(0x1)) ? '1' : '0',
-		(beacon->capability & BIT(0x0)) ? '1' : '0');
+		(beacon->capability & (1<<0xf)) ? '1' : '0',
+		(beacon->capability & (1<<0xe)) ? '1' : '0',
+		(beacon->capability & (1<<0xd)) ? '1' : '0',
+		(beacon->capability & (1<<0xc)) ? '1' : '0',
+		(beacon->capability & (1<<0xb)) ? '1' : '0',
+		(beacon->capability & (1<<0xa)) ? '1' : '0',
+		(beacon->capability & (1<<0x9)) ? '1' : '0',
+		(beacon->capability & (1<<0x8)) ? '1' : '0',
+		(beacon->capability & (1<<0x7)) ? '1' : '0',
+		(beacon->capability & (1<<0x6)) ? '1' : '0',
+		(beacon->capability & (1<<0x5)) ? '1' : '0',
+		(beacon->capability & (1<<0x4)) ? '1' : '0',
+		(beacon->capability & (1<<0x3)) ? '1' : '0',
+		(beacon->capability & (1<<0x2)) ? '1' : '0',
+		(beacon->capability & (1<<0x1)) ? '1' : '0',
+		(beacon->capability & (1<<0x0)) ? '1' : '0');
 
 	if (ieee80211_network_init(ieee, beacon, &network, stats)) {
 		IEEE80211_DEBUG_SCAN("Dropped '%s' (" MAC_FMT ") via %s.\n",
 				     escape_essid(info_element->data,
 						  info_element->len),
 				     MAC_ARG(beacon->header.addr3),
-				     WLAN_FC_GET_STYPE(beacon->header.frame_ctl) ==
+				     WLAN_FC_GET_STYPE(le16_to_cpu(beacon->header.frame_ctl)) == //IXP425
 				     IEEE80211_STYPE_PROBE_RESP ?
 				     "PROBE RESPONSE" : "BEACON");
 		return;
@@ -1117,6 +1117,9 @@ static inline void ieee80211_process_probe_response(
 
 	/* Search for this entry in the list and update it if it is
 	 * already there. */
+
+	spin_lock_irqsave(&ieee->lock, flags);
+
 	list_for_each_entry(target, &ieee->network_list, list) {
 		if (is_same_network(target, &network))
 			break;
@@ -1151,7 +1154,7 @@ static inline void ieee80211_process_probe_response(
 				     escape_essid(network.ssid,
 						  network.ssid_len),
 				     MAC_ARG(network.bssid),
-				     WLAN_FC_GET_STYPE(beacon->header.frame_ctl) ==
+				     WLAN_FC_GET_STYPE(le16_to_cpu(beacon->header.frame_ctl)) == //IXP425
 				     IEEE80211_STYPE_PROBE_RESP ?
 				     "PROBE RESPONSE" : "BEACON");
 #endif
@@ -1162,31 +1165,33 @@ static inline void ieee80211_process_probe_response(
 				     escape_essid(target->ssid,
 						  target->ssid_len),
 				     MAC_ARG(target->bssid),
-				     WLAN_FC_GET_STYPE(beacon->header.frame_ctl) ==
+				     WLAN_FC_GET_STYPE(le16_to_cpu(beacon->header.frame_ctl)) == //IXP425
 				     IEEE80211_STYPE_PROBE_RESP ?
 				     "PROBE RESPONSE" : "BEACON");
 		update_network(target, &network);
 	}
+
+	spin_unlock_irqrestore(&ieee->lock, flags);
 }
 
 void ieee80211_rx_mgt(struct ieee80211_device *ieee,
 		      struct ieee80211_hdr *header,
 		      struct ieee80211_rx_stats *stats)
 {
-	switch (WLAN_FC_GET_STYPE(header->frame_ctl)) {
+	switch (WLAN_FC_GET_STYPE(le16_to_cpu(header->frame_ctl))) { //IXP425
 	case IEEE80211_STYPE_ASSOC_RESP:
 		IEEE80211_DEBUG_MGMT("received ASSOCIATION RESPONSE (%d)\n",
-				     WLAN_FC_GET_STYPE(header->frame_ctl));
+				     WLAN_FC_GET_STYPE(le16_to_cpu(header->frame_ctl))); //IXP425
 		break;
 
 	case IEEE80211_STYPE_REASSOC_RESP:
 		IEEE80211_DEBUG_MGMT("received REASSOCIATION RESPONSE (%d)\n",
-				     WLAN_FC_GET_STYPE(header->frame_ctl));
+				     WLAN_FC_GET_STYPE(le16_to_cpu(header->frame_ctl))); //IXP425
 		break;
 
 	case IEEE80211_STYPE_PROBE_RESP:
 		IEEE80211_DEBUG_MGMT("received PROBE RESPONSE (%d)\n",
-				     WLAN_FC_GET_STYPE(header->frame_ctl));
+				     WLAN_FC_GET_STYPE(le16_to_cpu(header->frame_ctl))); //IXP425
 		IEEE80211_DEBUG_SCAN("Probe response\n");
 		ieee80211_process_probe_response(
 			ieee, (struct ieee80211_probe_response *)header, stats);
@@ -1194,7 +1199,7 @@ void ieee80211_rx_mgt(struct ieee80211_device *ieee,
 
 	case IEEE80211_STYPE_BEACON:
 		IEEE80211_DEBUG_MGMT("received BEACON (%d)\n",
-				     WLAN_FC_GET_STYPE(header->frame_ctl));
+				     WLAN_FC_GET_STYPE(le16_to_cpu(header->frame_ctl))); //IXP425
 		IEEE80211_DEBUG_SCAN("Beacon\n");
 		ieee80211_process_probe_response(
 			ieee, (struct ieee80211_probe_response *)header, stats);
@@ -1202,10 +1207,10 @@ void ieee80211_rx_mgt(struct ieee80211_device *ieee,
 
 	default:
 		IEEE80211_DEBUG_MGMT("received UNKNOWN (%d)\n",
-				     WLAN_FC_GET_STYPE(header->frame_ctl));
+				     WLAN_FC_GET_STYPE(le16_to_cpu(header->frame_ctl))); //IXP425
 		IEEE80211_WARNING("%s: Unknown management packet: %d\n",
 				  ieee->dev->name,
-				  WLAN_FC_GET_STYPE(header->frame_ctl));
+				  WLAN_FC_GET_STYPE(le16_to_cpu(header->frame_ctl))); //IXP425
 		break;
 	}
 }
